#include <msp430.h>
#include <stdint.h>
#include "dr_step_motor.h"
#include "fw_public.h"

#define CARRIER_FREQ     10000
#define TIMER_FULL_SCALE (SMCLK_FREQ / CARRIER_FREQ / 2) //ز

#define DRIVE_ENABLE_PORT_DIR   P7DIR |= BIT4  //趨DRV8833оƬʹܶ˿Ϊ
#define MOTOR_DRIVE_ENABLE      P7OUT |= BIT4  //DRV8833оƬʹ
#define MOTOR_DRIVE_DISABLE     P7OUT &= ~BIT4 //DRV8833оƬض

//APWMؼĴAPWM
#define PWM_APOS_CCR TA0CCR1
#define PWM_APOS_CTL TA0CCTL1
//AฺIO
#define ANEG_SET        P2OUT |= BIT2
#define ANEG_RST        P2OUT &= ~BIT2
//BIO
#define BPOS_SET        P2OUT |= BIT3
#define BPOS_RST        P2OUT &= ~BIT3
//BPWMؼĴBPWMڸ
#define PWM_BNEG_CCR TA0CCR3
#define PWM_BNEG_CTL TA0CCTL3

uint16_t step_per_tick = 0; //15λС
uint16_t cur_step = 0; //15λС

int32_t steps_to_move = 0;

void initStepMotor()
{
  //IOʼ
  MOTOR_DRIVE_DISABLE; //õ
  DRIVE_ENABLE_PORT_DIR; //ʹܶ˿
  
  //ʱ
  TA0CTL = TASSEL__SMCLK + MC__UPDOWN; //ز
  TA0CCR0 = TIMER_FULL_SCALE;
  TA0CCTL0 = CCIE; //10kHzж
  
  //IO趨
  P1SEL |= BIT2 + BIT4; //ѡʱ
  P1DIR |= BIT2 + BIT4; //
  P2OUT &= ~(BIT2 + BIT3);  //͵ƽ
  P2DIR |= BIT2 + BIT3; //
  //ϸģʽPWM
  PWM_APOS_CCR = 0; //AOUT1
  PWM_APOS_CTL = OUTMOD_5; //reset͵ƽ
  PWM_BNEG_CCR = 0; //BOUT2
  PWM_BNEG_CTL = OUTMOD_5; //reset͵ƽ
  
  //ʼ״̬
  StepMotor_MoveOneStep(1);
}

//ϸģʽ
#define STEP_LIST_SIZE 72 //еǶ1һǶ5һ
extern const int16_t STEP_LIST[STEP_LIST_SIZE * 2];
#include "dr_step_motor_list.h"

static inline int16_t StepMotor_CalTimerCnt(int16_t val)
{ //ӵѹֵöʱ趨ֵ
  int32_t temp = val;
  temp *= TIMER_FULL_SCALE;
  temp >>= 15;
  return temp;
}

static inline void StepMotor_SetVa(int16_t val) //APWM
{ //valΪѹֵ15λС
  val = StepMotor_CalTimerCnt(val);
  if(val >= 0) //Aѹ
  {
    PWM_APOS_CCR = (TIMER_FULL_SCALE + val) >> 1; //ȫ̬Чڼӷѹռձ50%Ӧʵ0V
    PWM_APOS_CTL = OUTMOD_2; //toggle/resetģʽvalԽߵƽԽЧѹԽ
    ANEG_RST; //͵ƽ
  }
  else //Aѹ
  {
    PWM_APOS_CCR = -val; //ȡþֵ
    PWM_APOS_CTL = OUTMOD_6; //toggle/setģʽvalԽС(ֵԽ)͵ƽʱԽЧѹԽ
    ANEG_SET; //ߵƽ
  }
}

static inline void StepMotor_SetVb(int16_t val) //BPWMڸ
{ //valΪѹֵ15λС
  val = StepMotor_CalTimerCnt(val);
  if(val >= 0) //Bѹ
  {
    BPOS_SET; //ߵƽ
    PWM_BNEG_CCR = val;
    PWM_BNEG_CTL = OUTMOD_6; //toggle/setģʽvalԽ͵ƽʱԽЧѹԽ
  }
  else //Bѹ
  {
    BPOS_RST; //͵ƽ
    PWM_BNEG_CCR = (TIMER_FULL_SCALE - val) >> 1; //ȫ̬Чڼӷѹռձ50%Ӧʵ0V
    PWM_BNEG_CTL = OUTMOD_2; //toggle/resetģʽvalԽС(ֵԽ)ߵƽʱԽЧѹԽ
  }
}

void StepMotor_MoveOneStep(int dir)
{
  static int step = -1;
  
  if(dir>=0)
    step++;
  else 
    step--;
  
  if(step < 0)
    step += STEP_LIST_SIZE;
  if(step >= STEP_LIST_SIZE)
    step -= STEP_LIST_SIZE;
  
  StepMotor_SetVa(STEP_LIST[step * 2]);
  StepMotor_SetVb(STEP_LIST[step * 2 + 1]);
}

void StepMotor_SetEnable(int enable)
{
  if(enable)
    MOTOR_DRIVE_ENABLE;
  else
    MOTOR_DRIVE_DISABLE;
}

void StepMotor_SetRPM(uint16_t rpm)
{
  uint16_t gie;
  _ECRIT(gie);
  step_per_tick = ((uint32_t)rpm) * (360L * 32768L / 60L) / CARRIER_FREQ; 
  _LCRIT(gie);
}

void StepMotor_AddStep(int32_t steps)
{
  uint16_t gie;
  _ECRIT(gie);
  steps_to_move += steps;
  _LCRIT(gie);
}

void StepMotor_ClearStep()
{
  uint16_t gie;
  _ECRIT(gie);
  steps_to_move = 0;
  _LCRIT(gie);
}

#pragma vector=TIMER0_A0_VECTOR
__interrupt void stepMotorInterrupt()
{
  cur_step += step_per_tick;
  if(cur_step >= 0x8000)
  {
    cur_step -= 0x8000;
    if(steps_to_move > 0)
    {
      steps_to_move--;
      StepMotor_MoveOneStep(1);
    }
    else if(steps_to_move < 0)
    {
      steps_to_move++;
      StepMotor_MoveOneStep(-1);
    }
  }
}
